@@ -5,15 +5,18 @@ library(sf)
 # print(nc, n = 3)
 # st_write(nc, "nc.gpkg", "nc.gpkg", driver = "GPKG")
 
+# description of the dataset, see vignette in package spdep:
+# https://cran.r-project.org/web/packages/spdep/vignettes/sids.pdf
+
 datasource = { if ("GPKG" %in% st_drivers()$name)
 	system.file("gpkg/nc.gpkg", package="sf")
 else
-	system.file("shapes/nc.shp", package="sf")
+	system.file("shape/nc.shp", package="sf")
 }
 
-nc = st_read(datasource, crs = 4267,
-	relation_to_geometry = c(AREA = "lattice", PERIMETER = "lattice", CNTY_ = "entity",
+rtg = c(AREA = "lattice", PERIMETER = "lattice", CNTY_ = "entity",
 		CNTY_ID = "entity", NAME = "entity", FIPS = "entity", FIPSNO = "entity",
 		CRESS_ID = "entity", BIR74 = "lattice", SID74 = "lattice", NWBIR74 = "lattice",
-		BIR79 = "lattice", SID79 = "lattice", NWBIR79  = "lattice"))
-rm(datasource)
+		BIR79 = "lattice", SID79 = "lattice", NWBIR79  = "lattice")
+
+nc = st_read(datasource, relation_to_geometry = rtg)
